unit uGNLContratoSuministro;

interface

uses uSuministroCombustible, uCosa;

type

  { TGNLContratoSuministro }

  (* Esta clase representará los contratos dentro de un suministro de combustible multicliente *)
  TGNLContratoSuministro = class(TCosa)

  public
    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    cliente: TSuministroCombustible;
    proveedor: TSuministroCombustible;

    (**************************************************************************)

    constructor Create(capa: integer; cli, prov: TSuministroCombustible);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    procedure Free; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
    {$ENDIF}

  published
    {$IFDEF MIGRACION_PERSISTENCIA}
    property _cliente:   TSuministroCombustible read cliente   write cliente;
    property _proveedor: TSuministroCombustible read proveedor write proveedor;
    {$ENDIF}
  end;

procedure AlInicio;
procedure AlFinal;

implementation

{ TGNLContratoSuministro }

constructor TGNLContratoSuministro.Create(capa: integer; cli, prov: TSuministroCombustible);
begin
  inherited Create( capa );
  cliente := cli;
  proveedor := prov;
end;


{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TGNLContratoSuministro.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rdReferencia('cliente', TCosa(cliente), self);
  f.rdReferencia('proveedor', TCosa(proveedor), self);
  //    f.rd('bornePreciosPorCombustible', bornePreciosPorCombustible);
  f.EjecutarLectura;
end;

procedure TGNLContratoSuministro.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wrReferencia('cliente', cliente);
  f.wrReferencia('proveedor', proveedor);
  //  f.wr('bornePreciosPorCombustible', bornePreciosPorCombustible);
end;
{$ENDIF}

procedure TGNLContratoSuministro.Free;
begin
  inherited Free;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TGNLContratoSuministro.CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddCosaColumn('cliente'  );
  Result.AddCosaColumn('proveedor');
end;
{$ENDIF}

procedure AlInicio;
begin
  registrarClaseDeCosa(TGNLContratoSuministro.ClassName, TGNLContratoSuministro);
end;

procedure AlFinal;
begin
end;



end.
