unit ugenpronogen;

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls,
  Graphics, Dialogs, ExtCtrls,
  StdCtrls,
  Menus, ComCtrls,
  xmatdefs, matreal,
  udbconpg,
  uparqueeolicodata;


const
  LONG1 = -60.50073333333333;
  LAT2 = -36.95866666666667;
  LONG2 = -51.914066666666656;
  LAT1 = -26.22533333333333;

  FactorDePerdida_V = 0.9; // para ajustar

type

  { TForm1 }

  TForm1 = class(TForm)
    btEstimarGen: TButton;
    btListarParques: TButton;
    Button1: TButton;
    Button2: TButton;
    btTablasScada: TButton;
    Memo1: TMemo;
    Shape1: TShape;
    StatusBar1: TStatusBar;
    procedure btEstimarGenClick(Sender: TObject);
    procedure btListarParquesClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure btTablasScadaClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    procedure InsertarCodigoMedida(Central: integer; Codigo: string;
      TipoMedida: integer);
    procedure InsertarTipoMedida(nombre, unidades: string);
    { private declarations }
  public
    { public declarations }

    dbpq: TDBPqCon;

  end;



var
  Form1: TForm1;

  nFilas, nCols: integer;
  Parques: TList;

implementation

{$R *.lfm}




function pad(k: integer; ndigitos: integer = 2): string;
var
  s: string;
begin
  s := IntToStr(k);
  while length(s) < ndigitos do
    s := '0' + s;
  Result := s;
end;

procedure TForm1.btEstimarGenClick(Sender: TObject);
var

  ds: TSQLQuery;

  kParque: integer;
  aParque: TParqueEolico;
  parques: Tlist;
  fsal: textfile;

  dt: TDateTime;
  NHoras, kHora: integer;
  acum, PMW: NReal;
  dtPrimeraMuestra: TDateTime;
  mb_data_Id: string;

begin
  ds := dbpq.query('SELECT * FROM centrales WHERE activo > 0 AND tipo = 0');
  parques:= TList.Create;
  dtPrimeraMuestra:= 0;
  while not ds.EOF do
  begin
    aParque := TParqueEolico.CreateFromDBRec( ds);
    parques.add( aParque );
writeln('Estimando parque: '+ aParque.CentralGeneradora );

    if aParque.CalcPronosticoGen( dbpq ) > 0 then
      if aParque.dt_PrimerHora > dtPrimeraMuestra then
        dtPrimeraMuestra:= aParque.dt_PrimerHora;
    ds.next;
  end;
  ds.Free;

  mb_data_id:= formatDateTime( 'yyyymmddhh', dtPrimeraMuestra );

  assignfile(fsal, 'c:\basura\prono_gen_' + mb_data_ID + '.xlt');
  rewrite(fsal);
  dt := aParque.dt_PrimerHora;
  writeln(fsal, 'Pronóstico de generación: ' + mb_data_ID, #9, 'Fecha: ',
    DateTimeToStr(now));
  writeln(fsal, 'Potencias en MW');
  NHoras := length(aParque.P);
  Write(fsal, '');
  for kParque := 0 to parques.count-1 do
    Write(fsal, #9, Utf8ToAnsi( TParqueEolico(parques[kParque]).CentralGeneradora));
  Write(fsal, #9, 'Total');
  writeln(fsal);

  for kHora := 0 to NHoras - 1 do
  begin
    Write(fsal, FormatDateTime('dd/mm/yyyy hh:nn', dt + kHora / 24));
    acum := 0;
    for kParque := 0 to parques.count-1 do
    begin
      if TParqueEolico(parques[kParque]).nidProno > 0 then
      begin
         PMW := TParqueEolico(parques[kParque]).P[kHora];
         Write(fsal, #9, PMW: 6: 1);
         acum := acum + PMW;
      end
      else
      begin
        Write(fsal, #9, '?' );
      end;
    end;
    writeln(fsal, #9, acum: 6: 1);
  end;

  closefile(fsal);

  for kParque := 0 to parques.count-1 do
    TParqueEolico(parques[kParque]).Free;
  parques.Free;

end;



procedure TForm1.btListarParquesClick(Sender: TObject);
var
  ds: TSQLQuery;
  aParque: TParqueEolico;
  fsal: textfile;

begin
  assignfile(fsal, 'c:\basura\ListadoParques.xlt');
  rewrite(fsal);
  TParqueEolico.WriteEncabToXLT(fsal);

  ds := dbpq.query('SELECT * FROM centrales ORDER BY activo DESC, nid ');
  while not ds.EOF do
  begin
    aParque := TParqueEolico.CreateFromDBRec( ds );
    aParque.WriteToXLT(fsal);
    aParque.Free;
    ds.next;
  end;
  ds.Free;
  closefile(fsal);
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  dbpq.exec( memo1.text );
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  r: TSQLQuery;
  j: integer;
begin
  r:= dbpq.query( memo1.text );
  while not r.EOF do
  begin
    writeln;
    for j:= 1 to r.nFields do
      write( ', ',r.Fields[j-1].AsString ) ;
    r.next;
  end;
  r.free;
end;


procedure TForm1.InsertarTipoMedida( nombre, unidades: string );
begin
  dbpq.exec( 'INSERT INTO tiposmedidasscada( nombre, unidades ) VALUES ('''+nombre+''', '''+unidades+''' ); ' );
end;

procedure TForm1.InsertarCodigoMedida( Central: integer; Codigo: string; TipoMedida: integer );
begin
  dbpq.exec( 'INSERT INTO codigosmedidasscada(tipomedida, central, codigo ) VALUES ( '
    +IntToStr( tipomedida)+', '+IntToStr( central )+', '''+codigo+''' ); ' );
end;

procedure TForm1.btTablasScadaClick(Sender: TObject);
begin
  (*
  sql_exec( 'CREATE TABLE tiposmedidasscada( '
   +' nid serial PRIMARY KEY,'
   +' nombre CHARACTER( 100 ), '
   +' unidades CHARACTER( 10 ) '
   +' ); ' );

  sql_exec( 'CREATE TABLE codigosmedidasscada('
   +' nid serial PRIMARY KEY,'
   +' tipomedida integer,'
   +' central integer,'
   +' codigo CHARACTER( 100 )'
   +'   );' );

InsertarTipoMedida('Gen_PActiva', 'MW');
InsertarTipoMedida('Ben_PReactiva', 'MVAR');
InsertarTipoMedida('MolinosDisponibles', 'u');
InsertarTipoMedida('ConsignaControl', 'MW');
InsertarTipoMedida('Temp_1', '°C');
InsertarTipoMedida('HumRel_1', '%');
InsertarTipoMedida('Presion_1', 'hPA');
InsertarTipoMedida('VelVIento_1', 'm/s');
InsertarTipoMedida('DireViento_1', '°');
InsertarTipoMedida('RadSol_1', 'W/m2');
InsertarTipoMedida('Temp_2', '°C');
InsertarTipoMedida('HumRel_2', '%');
InsertarTipoMedida('Presion_2', 'hPA');
InsertarTipoMedida('VelVIento_2', 'm/s');
InsertarTipoMedida('DireViento_2', '°');
InsertarTipoMedida('RadSol_2', 'W/m2');
InsertarTipoMedida('PCM_PActiva', 'MW');
InsertarTipoMedida('PCM_PReactiva', 'MVAR');



  InsertarCodigoMedida( 5, 'X', 2 ); 	InsertarCodigoMedida( 5, 'X', 3 ); 	InsertarCodigoMedida( 5, 'X', 4 ); 	InsertarCodigoMedida( 5, 'X', 5 ); 	InsertarCodigoMedida( 5, 'X', 6 ); 	InsertarCodigoMedida( 5, 'X', 7 ); 	InsertarCodigoMedida( 5, 'X', 8 ); 	InsertarCodigoMedida( 5, 'X', 9 ); 	InsertarCodigoMedida( 5, 'X', 10 ); 	InsertarCodigoMedida( 5, 'X', 11 ); 	InsertarCodigoMedida( 5, 'X', 12 ); 	InsertarCodigoMedida( 5, 'X', 13 ); 	InsertarCodigoMedida( 5, 'X', 14 ); 	InsertarCodigoMedida( 5, 'X', 15 ); 	InsertarCodigoMedida( 5, 'X', 16 ); 	InsertarCodigoMedida( 5, 'X', 17 ); 	InsertarCodigoMedida( 5, 'X', 18 );
  InsertarCodigoMedida( 8, 'X', 2 ); 	InsertarCodigoMedida( 8, 'X', 3 ); 	InsertarCodigoMedida( 8, 'X', 4 ); 	InsertarCodigoMedida( 8, 'X', 5 ); 	InsertarCodigoMedida( 8, 'X', 6 ); 	InsertarCodigoMedida( 8, 'X', 7 ); 	InsertarCodigoMedida( 8, 'X', 8 ); 	InsertarCodigoMedida( 8, 'X', 9 ); 	InsertarCodigoMedida( 8, 'X', 10 ); 	InsertarCodigoMedida( 8, 'X', 11 ); 	InsertarCodigoMedida( 8, 'X', 12 ); 	InsertarCodigoMedida( 8, 'X', 13 ); 	InsertarCodigoMedida( 8, 'X', 14 ); 	InsertarCodigoMedida( 8, 'X', 15 ); 	InsertarCodigoMedida( 8, 'X', 16 ); 	InsertarCodigoMedida( 8, 'X', 17 ); 	InsertarCodigoMedida( 8, 'X', 18 );
  InsertarCodigoMedida( 39, 'X', 2 ); 	InsertarCodigoMedida( 39, 'X', 3 ); 	InsertarCodigoMedida( 39, 'X', 4 ); 	InsertarCodigoMedida( 39, 'X', 5 ); 	InsertarCodigoMedida( 39, 'X', 6 ); 	InsertarCodigoMedida( 39, 'X', 7 ); 	InsertarCodigoMedida( 39, 'X', 8 ); 	InsertarCodigoMedida( 39, 'X', 9 ); 	InsertarCodigoMedida( 39, 'X', 10 ); 	InsertarCodigoMedida( 39, 'X', 11 ); 	InsertarCodigoMedida( 39, 'X', 12 ); 	InsertarCodigoMedida( 39, 'X', 13 ); 	InsertarCodigoMedida( 39, 'X', 14 ); 	InsertarCodigoMedida( 39, 'X', 15 ); 	InsertarCodigoMedida( 39, 'X', 16 ); 	InsertarCodigoMedida( 39, 'X', 17 ); 	InsertarCodigoMedida( 39, 'X', 18 );
  InsertarCodigoMedida( 10, 'X', 2 ); 	InsertarCodigoMedida( 10, 'X', 3 ); 	InsertarCodigoMedida( 10, 'X', 4 ); 	InsertarCodigoMedida( 10, 'X', 5 ); 	InsertarCodigoMedida( 10, 'X', 6 ); 	InsertarCodigoMedida( 10, 'X', 7 ); 	InsertarCodigoMedida( 10, 'X', 8 ); 	InsertarCodigoMedida( 10, 'X', 9 ); 	InsertarCodigoMedida( 10, 'X', 10 ); 	InsertarCodigoMedida( 10, 'X', 11 ); 	InsertarCodigoMedida( 10, 'X', 12 ); 	InsertarCodigoMedida( 10, 'X', 13 ); 	InsertarCodigoMedida( 10, 'X', 14 ); 	InsertarCodigoMedida( 10, 'X', 15 ); 	InsertarCodigoMedida( 10, 'X', 16 ); 	InsertarCodigoMedida( 10, 'X', 17 ); 	InsertarCodigoMedida( 10, 'X', 18 );
  InsertarCodigoMedida( 11, 'PERALTA1.030 GEN PE1 MED Q       .AV', 2 ); 	InsertarCodigoMedida( 11, '', 3 ); 	InsertarCodigoMedida( 11, 'PERALTA1.000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 11, 'X', 5 ); 	InsertarCodigoMedida( 11, 'X', 6 ); 	InsertarCodigoMedida( 11, 'X', 7 ); 	InsertarCodigoMedida( 11, 'X', 8 ); 	InsertarCodigoMedida( 11, 'X', 9 ); 	InsertarCodigoMedida( 11, 'X', 10 ); 	InsertarCodigoMedida( 11, 'X', 11 ); 	InsertarCodigoMedida( 11, 'X', 12 ); 	InsertarCodigoMedida( 11, 'X', 13 ); 	InsertarCodigoMedida( 11, 'X', 14 ); 	InsertarCodigoMedida( 11, 'X', 15 ); 	InsertarCodigoMedida( 11, 'X', 16 ); 	InsertarCodigoMedida( 11, 'X', 17 ); 	InsertarCodigoMedida( 11, 'X', 18 );
  InsertarCodigoMedida( 12, 'PERALTA2.030 GEN PE2 MED Q       .AV', 2 ); 	InsertarCodigoMedida( 12, '', 3 ); 	InsertarCodigoMedida( 12, 'PERALTA2.000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 12, 'X', 5 ); 	InsertarCodigoMedida( 12, 'X', 6 ); 	InsertarCodigoMedida( 12, 'X', 7 ); 	InsertarCodigoMedida( 12, 'X', 8 ); 	InsertarCodigoMedida( 12, 'X', 9 ); 	InsertarCodigoMedida( 12, 'X', 10 ); 	InsertarCodigoMedida( 12, 'X', 11 ); 	InsertarCodigoMedida( 12, 'X', 12 ); 	InsertarCodigoMedida( 12, 'X', 13 ); 	InsertarCodigoMedida( 12, 'X', 14 ); 	InsertarCodigoMedida( 12, 'X', 15 ); 	InsertarCodigoMedida( 12, 'X', 16 ); 	InsertarCodigoMedida( 12, 'X', 17 ); 	InsertarCodigoMedida( 12, 'X', 18 );
  InsertarCodigoMedida( 13, 'TMACIEL1.150 TRA TR1 MED Q       .AV', 2 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 CANT MOLINOS DISPON .AV', 3 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 TEMPERATURA  12M 4  .AV', 5 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 HUMEDAD 12M 4       .AV', 6 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 PRESION 12M 4       .AV', 7 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 VEL VIENTO 65M 2    .AV', 8 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 DIR VIENTO 88M 1    .AV', 9 ); 	InsertarCodigoMedida( 13, 'X', 10 ); 	InsertarCodigoMedida( 13, 'X', 11 ); 	InsertarCodigoMedida( 13, 'X', 12 ); 	InsertarCodigoMedida( 13, 'X', 13 ); 	InsertarCodigoMedida( 13, 'TMACIEL1.000 VEL VIENTO 90M 1    .AV', 14 ); 	InsertarCodigoMedida( 13, 'X', 15 ); 	InsertarCodigoMedida( 13, 'X', 16 ); 	InsertarCodigoMedida( 13, 'X', 17 ); 	InsertarCodigoMedida( 13, 'X', 18 );
  InsertarCodigoMedida( 14, 'TMACIEL2.150 TRA TR1 MED Q       .AV', 2 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 CANT MOLINOS DISPON .AV', 3 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 TEMPERATURA  ALT1   .AV', 5 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 HUMEDAD ALT1        .AV', 6 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 PRESION ALT1        .AV', 7 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 VEL VIENTO ALT1     .AV', 8 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 DIR VIENTO ALT1     .AV', 9 ); 	InsertarCodigoMedida( 14, 'X', 10 ); 	InsertarCodigoMedida( 14, 'X', 11 ); 	InsertarCodigoMedida( 14, 'X', 12 ); 	InsertarCodigoMedida( 14, 'X', 13 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 VEL VIENTO ALT2     .AV', 14 ); 	InsertarCodigoMedida( 14, 'TMACIEL2.000 DIR VIENTO ALT2     .AV', 15 ); 	InsertarCodigoMedida( 14, 'X', 16 ); 	InsertarCodigoMedida( 14, 'X', 17 ); 	InsertarCodigoMedida( 14, 'X', 18 );
  InsertarCodigoMedida( 15, 'X', 2 ); 	InsertarCodigoMedida( 15, 'X', 3 ); 	InsertarCodigoMedida( 15, 'X', 4 ); 	InsertarCodigoMedida( 15, 'X', 5 ); 	InsertarCodigoMedida( 15, 'X', 6 ); 	InsertarCodigoMedida( 15, 'X', 7 ); 	InsertarCodigoMedida( 15, 'X', 8 ); 	InsertarCodigoMedida( 15, 'X', 9 ); 	InsertarCodigoMedida( 15, 'X', 10 ); 	InsertarCodigoMedida( 15, 'X', 11 ); 	InsertarCodigoMedida( 15, 'X', 12 ); 	InsertarCodigoMedida( 15, 'X', 13 ); 	InsertarCodigoMedida( 15, 'X', 14 ); 	InsertarCodigoMedida( 15, 'X', 15 ); 	InsertarCodigoMedida( 15, 'X', 16 ); 	InsertarCodigoMedida( 15, 'PCARACOL.030 GEN TOT MED P       .AV', 17 ); 	InsertarCodigoMedida( 15, 'X', 18 );
  InsertarCodigoMedida( 16, 'X', 2 ); 	InsertarCodigoMedida( 16, 'X', 3 ); 	InsertarCodigoMedida( 16, 'X', 4 ); 	InsertarCodigoMedida( 16, 'X', 5 ); 	InsertarCodigoMedida( 16, 'X', 6 ); 	InsertarCodigoMedida( 16, 'X', 7 ); 	InsertarCodigoMedida( 16, 'X', 8 ); 	InsertarCodigoMedida( 16, 'X', 9 ); 	InsertarCodigoMedida( 16, 'X', 10 ); 	InsertarCodigoMedida( 16, 'X', 11 ); 	InsertarCodigoMedida( 16, 'X', 12 ); 	InsertarCodigoMedida( 16, 'X', 13 ); 	InsertarCodigoMedida( 16, 'X', 14 ); 	InsertarCodigoMedida( 16, 'X', 15 ); 	InsertarCodigoMedida( 16, 'X', 16 ); 	InsertarCodigoMedida( 16, 'X', 17 ); 	InsertarCodigoMedida( 16, 'X', 18 );
  InsertarCodigoMedida( 17, 'X', 2 ); 	InsertarCodigoMedida( 17, 'X', 3 ); 	InsertarCodigoMedida( 17, 'X', 4 ); 	InsertarCodigoMedida( 17, 'X', 5 ); 	InsertarCodigoMedida( 17, 'X', 6 ); 	InsertarCodigoMedida( 17, 'X', 7 ); 	InsertarCodigoMedida( 17, 'X', 8 ); 	InsertarCodigoMedida( 17, 'X', 9 ); 	InsertarCodigoMedida( 17, 'X', 10 ); 	InsertarCodigoMedida( 17, 'X', 11 ); 	InsertarCodigoMedida( 17, 'X', 12 ); 	InsertarCodigoMedida( 17, 'X', 13 ); 	InsertarCodigoMedida( 17, 'X', 14 ); 	InsertarCodigoMedida( 17, 'X', 15 ); 	InsertarCodigoMedida( 17, 'X', 16 ); 	InsertarCodigoMedida( 17, 'X', 17 ); 	InsertarCodigoMedida( 17, 'X', 18 );
  InsertarCodigoMedida( 20, 'LUZMAR  .030 LIN PIB MED Q       .AV', 2 ); 	InsertarCodigoMedida( 20, 'LUZMAR  .000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 20, 'LUZMAR  .000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 20, 'LUZMAR  .000 TEMPERATURA 80M     .AV', 5 ); 	InsertarCodigoMedida( 20, 'X', 6 ); 	InsertarCodigoMedida( 20, 'X', 7 ); 	InsertarCodigoMedida( 20, 'LUZMAR  .000 VEL VIENTO 80M      .AV', 8 ); 	InsertarCodigoMedida( 20, 'LUZMAR  .000 DIR VIENTO 80M      .AV', 9 ); 	InsertarCodigoMedida( 20, 'X', 10 ); 	InsertarCodigoMedida( 20, 'X', 11 ); 	InsertarCodigoMedida( 20, 'X', 12 ); 	InsertarCodigoMedida( 20, 'X', 13 ); 	InsertarCodigoMedida( 20, 'X', 14 ); 	InsertarCodigoMedida( 20, 'X', 15 ); 	InsertarCodigoMedida( 20, 'X', 16 ); 	InsertarCodigoMedida( 20, 'X', 17 ); 	InsertarCodigoMedida( 20, 'X', 18 );
  InsertarCodigoMedida( 21, 'LUZRIO  .030 LIN PIB MED Q       .AV', 2 ); 	InsertarCodigoMedida( 21, 'LUZRIO  .000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 21, 'LUZRIO  .000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 21, 'LUZRIO  .000 TEMPERATURA 80M     .AV', 5 ); 	InsertarCodigoMedida( 21, 'X', 6 ); 	InsertarCodigoMedida( 21, 'X', 7 ); 	InsertarCodigoMedida( 21, 'LUZRIO  .000 VEL VIENTO 80M      .AV', 8 ); 	InsertarCodigoMedida( 21, 'LUZRIO  .000 DIR VIENTO 80M      .AV', 9 ); 	InsertarCodigoMedida( 21, 'X', 10 ); 	InsertarCodigoMedida( 21, 'X', 11 ); 	InsertarCodigoMedida( 21, 'X', 12 ); 	InsertarCodigoMedida( 21, 'X', 13 ); 	InsertarCodigoMedida( 21, 'X', 14 ); 	InsertarCodigoMedida( 21, 'X', 15 ); 	InsertarCodigoMedida( 21, 'X', 16 ); 	InsertarCodigoMedida( 21, 'X', 17 ); 	InsertarCodigoMedida( 21, 'X', 18 );
  InsertarCodigoMedida( 28, 'LUZLOMA .030 LIN PIB MED Q       .AV', 2 ); 	InsertarCodigoMedida( 28, 'LUZLOMA .000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 28, 'LUZLOMA .030 GEN TOT MED P       .AV', 4 ); 	InsertarCodigoMedida( 28, 'LUZLOMA .000 TEMPERATURA 80M     .AV', 5 ); 	InsertarCodigoMedida( 28, 'X', 6 ); 	InsertarCodigoMedida( 28, 'X', 7 ); 	InsertarCodigoMedida( 28, 'LUZLOMA .000 VEL VIENTO 80M      .AV', 8 ); 	InsertarCodigoMedida( 28, 'LUZLOMA .000 DIR VIENTO 80M      .AV', 9 ); 	InsertarCodigoMedida( 28, 'X', 10 ); 	InsertarCodigoMedida( 28, 'X', 11 ); 	InsertarCodigoMedida( 28, 'X', 12 ); 	InsertarCodigoMedida( 28, 'X', 13 ); 	InsertarCodigoMedida( 28, 'X', 14 ); 	InsertarCodigoMedida( 28, 'X', 15 ); 	InsertarCodigoMedida( 28, 'X', 16 ); 	InsertarCodigoMedida( 28, 'X', 17 ); 	InsertarCodigoMedida( 28, 'X', 18 );
  InsertarCodigoMedida( 22, 'X', 2 ); 	InsertarCodigoMedida( 22, 'X', 3 ); 	InsertarCodigoMedida( 22, 'X', 4 ); 	InsertarCodigoMedida( 22, 'X', 5 ); 	InsertarCodigoMedida( 22, 'X', 6 ); 	InsertarCodigoMedida( 22, 'X', 7 ); 	InsertarCodigoMedida( 22, 'X', 8 ); 	InsertarCodigoMedida( 22, 'X', 9 ); 	InsertarCodigoMedida( 22, 'X', 10 ); 	InsertarCodigoMedida( 22, 'X', 11 ); 	InsertarCodigoMedida( 22, 'X', 12 ); 	InsertarCodigoMedida( 22, 'X', 13 ); 	InsertarCodigoMedida( 22, 'X', 14 ); 	InsertarCodigoMedida( 22, 'X', 15 ); 	InsertarCodigoMedida( 22, 'X', 16 ); 	InsertarCodigoMedida( 22, 'NVOMANAN.060 GEN G02 MED P       .AV', 17 ); 	InsertarCodigoMedida( 22, 'X', 18 );
  InsertarCodigoMedida( 23, 'ELIBERTA.030 GEN ELI MED Q       .AV', 2 ); 	InsertarCodigoMedida( 23, 'ELIBERTA.000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 23, 'ELIBERTA.000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 23, 'ELIBERTA.000 TEMPERATURA         .AV', 5 ); 	InsertarCodigoMedida( 23, 'X', 6 ); 	InsertarCodigoMedida( 23, 'X', 7 ); 	InsertarCodigoMedida( 23, 'ELIBERTA.000 VEL VIENTO          .AV', 8 ); 	InsertarCodigoMedida( 23, 'ELIBERTA.000 DIR VIENTO          .AV', 9 ); 	InsertarCodigoMedida( 23, 'X', 10 ); 	InsertarCodigoMedida( 23, 'X', 11 ); 	InsertarCodigoMedida( 23, 'X', 12 ); 	InsertarCodigoMedida( 23, 'X', 13 ); 	InsertarCodigoMedida( 23, 'X', 14 ); 	InsertarCodigoMedida( 23, 'X', 15 ); 	InsertarCodigoMedida( 23, 'X', 16 ); 	InsertarCodigoMedida( 23, 'X', 17 ); 	InsertarCodigoMedida( 23, 'X', 18 );
  InsertarCodigoMedida( 24, 'X', 2 ); 	InsertarCodigoMedida( 24, 'X', 3 ); 	InsertarCodigoMedida( 24, 'X', 4 ); 	InsertarCodigoMedida( 24, 'X', 5 ); 	InsertarCodigoMedida( 24, 'X', 6 ); 	InsertarCodigoMedida( 24, 'X', 7 ); 	InsertarCodigoMedida( 24, 'X', 8 ); 	InsertarCodigoMedida( 24, 'X', 9 ); 	InsertarCodigoMedida( 24, 'X', 10 ); 	InsertarCodigoMedida( 24, 'X', 11 ); 	InsertarCodigoMedida( 24, 'X', 12 ); 	InsertarCodigoMedida( 24, 'X', 13 ); 	InsertarCodigoMedida( 24, 'X', 14 ); 	InsertarCodigoMedida( 24, 'X', 15 ); 	InsertarCodigoMedida( 24, 'X', 16 ); 	InsertarCodigoMedida( 24, 'NVOMANAN.060 GEN G01 MED P       .AV', 17 ); 	InsertarCodigoMedida( 24, 'X', 18 );
  InsertarCodigoMedida( 25, 'X', 2 ); 	InsertarCodigoMedida( 25, 'X', 3 ); 	InsertarCodigoMedida( 25, 'X', 4 ); 	InsertarCodigoMedida( 25, 'X', 5 ); 	InsertarCodigoMedida( 25, 'X', 6 ); 	InsertarCodigoMedida( 25, 'X', 7 ); 	InsertarCodigoMedida( 25, 'X', 8 ); 	InsertarCodigoMedida( 25, 'X', 9 ); 	InsertarCodigoMedida( 25, 'X', 10 ); 	InsertarCodigoMedida( 25, 'X', 11 ); 	InsertarCodigoMedida( 25, 'X', 12 ); 	InsertarCodigoMedida( 25, 'X', 13 ); 	InsertarCodigoMedida( 25, 'X', 14 ); 	InsertarCodigoMedida( 25, 'X', 15 ); 	InsertarCodigoMedida( 25, 'X', 16 ); 	InsertarCodigoMedida( 25, 'X', 17 ); 	InsertarCodigoMedida( 25, 'X', 18 );
  InsertarCodigoMedida( 26, 'RDELSUR .000 GEN MOL MED Q       .AV', 2 ); 	InsertarCodigoMedida( 26, 'RDELSUR .000 CANT MOLINOS DISPON .AV', 3 ); 	InsertarCodigoMedida( 26, 'RDELSUR .000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 26, 'RDELSUR .000 TEMP 80M G16        .AV', 5 ); 	InsertarCodigoMedida( 26, 'X', 6 ); 	InsertarCodigoMedida( 26, 'X', 7 ); 	InsertarCodigoMedida( 26, 'RDELSUR .000 VEL VIENTO 80M G16  .AV', 8 ); 	InsertarCodigoMedida( 26, 'X', 9 ); 	InsertarCodigoMedida( 26, 'X', 10 ); 	InsertarCodigoMedida( 26, 'X', 11 ); 	InsertarCodigoMedida( 26, 'X', 12 ); 	InsertarCodigoMedida( 26, 'X', 13 ); 	InsertarCodigoMedida( 26, 'RDELSUR .000 VEL VIENTO 30M G16  .AV', 14 ); 	InsertarCodigoMedida( 26, 'X', 15 ); 	InsertarCodigoMedida( 26, 'X', 16 ); 	InsertarCodigoMedida( 26, 'X', 17 ); 	InsertarCodigoMedida( 26, 'X', 18 );
  InsertarCodigoMedida( 27, 'X', 2 ); 	InsertarCodigoMedida( 27, 'X', 3 ); 	InsertarCodigoMedida( 27, 'X', 4 ); 	InsertarCodigoMedida( 27, 'X', 5 ); 	InsertarCodigoMedida( 27, 'X', 6 ); 	InsertarCodigoMedida( 27, 'X', 7 ); 	InsertarCodigoMedida( 27, 'X', 8 ); 	InsertarCodigoMedida( 27, 'X', 9 ); 	InsertarCodigoMedida( 27, 'X', 10 ); 	InsertarCodigoMedida( 27, 'X', 11 ); 	InsertarCodigoMedida( 27, 'X', 12 ); 	InsertarCodigoMedida( 27, 'X', 13 ); 	InsertarCodigoMedida( 27, 'X', 14 ); 	InsertarCodigoMedida( 27, 'X', 15 ); 	InsertarCodigoMedida( 27, 'X', 16 ); 	InsertarCodigoMedida( 27, 'X', 17 ); 	InsertarCodigoMedida( 27, 'X', 18 );
  InsertarCodigoMedida( 29, 'EMINAS1 .030 GEN GEM MED Q       .AV', 2 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 TEMPERATURA 82M     .AV', 5 ); 	InsertarCodigoMedida( 29, 'X', 6 ); 	InsertarCodigoMedida( 29, 'X', 7 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 VEL VIENTO 100M     .AV', 8 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 DIR VIENTO 82M      .AV', 9 ); 	InsertarCodigoMedida( 29, 'X', 10 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 TEMPERATURA 10M     .AV', 11 ); 	InsertarCodigoMedida( 29, 'X', 12 ); 	InsertarCodigoMedida( 29, 'X', 13 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 VEL VIENTO 84M      .AV', 14 ); 	InsertarCodigoMedida( 29, 'EMINAS1 .000 DIR VIENTO 98M      .AV', 15 ); 	InsertarCodigoMedida( 29, 'X', 16 ); 	InsertarCodigoMedida( 29, 'X', 17 ); 	InsertarCodigoMedida( 29, 'X', 18 );
  InsertarCodigoMedida( 30, 'EFLOR1  .030 TRA TR1 MED Q       .AV', 2 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 TEMPERATURA 86M     .AV', 5 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 HUMEDAD             .AV', 6 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 PRESION             .AV', 7 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 VEL VIENTO 89M      .AV', 8 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 DIR VIENTO 85M 1    .AV', 9 ); 	InsertarCodigoMedida( 30, 'X', 10 ); 	InsertarCodigoMedida( 30, 'X', 11 ); 	InsertarCodigoMedida( 30, 'X', 12 ); 	InsertarCodigoMedida( 30, 'X', 13 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 VEL VIENTO 91M      .AV', 14 ); 	InsertarCodigoMedida( 30, 'EFLOR1  .000 DIR VIENTO 85M 2    .AV', 15 ); 	InsertarCodigoMedida( 30, 'X', 16 ); 	InsertarCodigoMedida( 30, 'X', 17 ); 	InsertarCodigoMedida( 30, 'X', 18 );
  InsertarCodigoMedida( 33, 'JPTERRA .030 GEN EJT MED Q       .AV', 2 ); 	InsertarCodigoMedida( 33, 'JPTERRA .000 CANT MOLINOS DISP   .AV', 3 ); 	InsertarCodigoMedida( 33, 'JPTERRA .000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 33, 'JPTERRA .000 TEMPERATURA 80M     .AV', 5 ); 	InsertarCodigoMedida( 33, 'X', 6 ); 	InsertarCodigoMedida( 33, 'X', 7 ); 	InsertarCodigoMedida( 33, 'JPTERRA .000 VEL VIENTO 80M      .AV', 8 ); 	InsertarCodigoMedida( 33, 'JPTERRA .000 DIR VIENTO 80M      .AV', 9 ); 	InsertarCodigoMedida( 33, 'X', 10 ); 	InsertarCodigoMedida( 33, 'X', 11 ); 	InsertarCodigoMedida( 33, 'X', 12 ); 	InsertarCodigoMedida( 33, 'X', 13 ); 	InsertarCodigoMedida( 33, 'X', 14 ); 	InsertarCodigoMedida( 33, 'X', 15 ); 	InsertarCodigoMedida( 33, 'X', 16 ); 	InsertarCodigoMedida( 33, 'X', 17 ); 	InsertarCodigoMedida( 33, 'X', 18 );
  InsertarCodigoMedida( 36, 'X', 2 ); 	InsertarCodigoMedida( 36, 'X', 3 ); 	InsertarCodigoMedida( 36, 'X', 4 ); 	InsertarCodigoMedida( 36, 'X', 5 ); 	InsertarCodigoMedida( 36, 'X', 6 ); 	InsertarCodigoMedida( 36, 'X', 7 ); 	InsertarCodigoMedida( 36, 'X', 8 ); 	InsertarCodigoMedida( 36, 'X', 9 ); 	InsertarCodigoMedida( 36, 'X', 10 ); 	InsertarCodigoMedida( 36, 'X', 11 ); 	InsertarCodigoMedida( 36, 'X', 12 ); 	InsertarCodigoMedida( 36, 'X', 13 ); 	InsertarCodigoMedida( 36, 'X', 14 ); 	InsertarCodigoMedida( 36, 'X', 15 ); 	InsertarCodigoMedida( 36, 'X', 16 ); 	InsertarCodigoMedida( 36, 'X', 17 ); 	InsertarCodigoMedida( 36, 'X', 18 );
  InsertarCodigoMedida( 38, 'X', 2 ); 	InsertarCodigoMedida( 38, 'X', 3 ); 	InsertarCodigoMedida( 38, 'X', 4 ); 	InsertarCodigoMedida( 38, 'X', 5 ); 	InsertarCodigoMedida( 38, 'X', 6 ); 	InsertarCodigoMedida( 38, 'X', 7 ); 	InsertarCodigoMedida( 38, 'X', 8 ); 	InsertarCodigoMedida( 38, 'X', 9 ); 	InsertarCodigoMedida( 38, 'X', 10 ); 	InsertarCodigoMedida( 38, 'X', 11 ); 	InsertarCodigoMedida( 38, 'X', 12 ); 	InsertarCodigoMedida( 38, 'X', 13 ); 	InsertarCodigoMedida( 38, 'X', 14 ); 	InsertarCodigoMedida( 38, 'X', 15 ); 	InsertarCodigoMedida( 38, 'X', 16 ); 	InsertarCodigoMedida( 38, 'X', 17 ); 	InsertarCodigoMedida( 38, 'X', 18 );
  InsertarCodigoMedida( 18, 'X', 2 ); 	InsertarCodigoMedida( 18, 'X', 3 ); 	InsertarCodigoMedida( 18, 'X', 4 ); 	InsertarCodigoMedida( 18, 'X', 5 ); 	InsertarCodigoMedida( 18, 'X', 6 ); 	InsertarCodigoMedida( 18, 'X', 7 ); 	InsertarCodigoMedida( 18, 'X', 8 ); 	InsertarCodigoMedida( 18, 'X', 9 ); 	InsertarCodigoMedida( 18, 'X', 10 ); 	InsertarCodigoMedida( 18, 'X', 11 ); 	InsertarCodigoMedida( 18, 'X', 12 ); 	InsertarCodigoMedida( 18, 'X', 13 ); 	InsertarCodigoMedida( 18, 'X', 14 ); 	InsertarCodigoMedida( 18, 'X', 15 ); 	InsertarCodigoMedida( 18, 'X', 16 ); 	InsertarCodigoMedida( 18, 'X', 17 ); 	InsertarCodigoMedida( 18, 'X', 18 );
  InsertarCodigoMedida( 19, 'X', 2 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 CANT MOLINOS DISPON .AV', 3 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 CONSIGNA P          .AV', 4 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 TEMP 101M           .AV', 5 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 HUMEDAD 100M        .AV', 6 ); 	InsertarCodigoMedida( 19, 'X', 7 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 VEL VIENTO 100M     .AV', 8 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 DIR VIENTO 100M     .AV', 9 ); 	InsertarCodigoMedida( 19, 'X', 10 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 HUMEDAD 100M        .AV', 11 ); 	InsertarCodigoMedida( 19, 'X', 12 ); 	InsertarCodigoMedida( 19, 'X', 13 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 VEL VIENTO 30M G1   .AV', 14 ); 	InsertarCodigoMedida( 19, 'PALMATIR.000 DIR VIENTO 30M G1   .AV', 15 ); 	InsertarCodigoMedida( 19, 'X', 16 ); 	InsertarCodigoMedida( 19, '', 17 ); 	InsertarCodigoMedida( 19, '', 18 );

  sql_exec( 'ALTER TABLE "codigosmedidasscada" ADD COLUMN "dtprimeramuestra" timestamp without time zone DEFAULT ''1900-01-01 00:00''' );
  sql_exec( 'ALTER TABLE "codigosmedidasscada" ADD COLUMN "dtultimamuestra" timestamp without time zone DEFAULT ''1900-01-01 00:00''' );
  sql_exec( 'ALTER TABLE "codigosmedidasscada" ADD COLUMN "estado" integer DEFAULT 0' );
  sql_exec( 'UPDATE codigosmedidasscada SET estado = 1 WHERE codigo <> ''X'';' );
  *)
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  if dbpq <> nil then dbpq.Free;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  dbpq:=  TDBPQCon.Create( 'condb.dat' );
end;


end.
