unit ufraccionadorcegh;

interface

uses
  Classes, SysUtils,
  umodelosintcegh,
  matreal, xMatDefs, fddp;


// Programa principal del Fraccionador que toma un archivo CEGH y un factor
// de Fraccionacion y calcula las Matrices A y B del filtro asi como las
// funciones deformantes del nuevo sintetizador
procedure Fraccionar(paso_discr: integer; arch: string);


// Funcion para interpolar las funciones deformantes cuando se Fracciona
function Interpolacion_funciones_deformantes(alf, Inferior, Superior: NReal): NReal;

// Procedimiento auxiliar para mostrar los datos del sintetizador origen
procedure muestra(arch: string; var a, b, c: integer);


implementation

function Interpolacion_funciones_deformantes(alf, Inferior, Superior: NReal): NReal;
begin
  Result := Inferior; //alf*Inferior+(1-alf)*Superior+(Superior-Inferior)/2;
end;




function TrasnformarB( MA, MB: TMatR; n: integer): TMatR;
var
  M,
  Aj,
  BBT: TMatR;
  VecBBT: TvectR;
  mVecBBT: TMatR;

  i: integer;
  aux: TMatR;
  invertible: boolean;
  exp10: integer;
  res: TMatR;

begin
  Aj := TMatR.Create_Identidad( MA.nf );
  M := TMatR.Create_kron(Aj, Aj);
  for i := 1 to n - 1 do
  begin
    Aj.Mult(Aj, MA );
    aux := TMatR.Create_Kron( Aj, Aj );
    M.suma( M, aux);
    aux.Free;
  end;
  Aj.Free;

  BBT := MB.Crear_Transpuesta;
  BBT.Mult(MB, aux);
  VecBBT := BBT.vec;
  mVecBBT:= VecBBT.reshape( VecBBT.n, 1 );

  M.Escaler( mVecBBT, invertible, exp10 );
  if not invertible then
     raise Exception.Create( 'ERror, no pude invertir sum( kron( aj^j, aj^j; j= 0..n-1 )' );
  mVecBBT.CopyColVect( VecBBT, 1);
  mVecBBT.Free;
  M.Free;

  BBT.Free;
  BBT:= VecBBT.reshape( MB.nf, MB.nc );
  res:= BBT.raiz_Cholesky;
  BBT.Free;
  VecBBT.Free;
  Result := res;

end;


procedure Fraccionar(paso_discr: integer; arch: string);
var
  sintetizador_origen, sintetizador_destino: TModeloSintetizadorCEGH;
  arch_salida: string;
  MatA, MatB: TMatR;
  NSS_origen, NFD_origen, NPFD_origen, NSS_destino, NFD_destino,
  NPFD_destino, kserie, kpaso, kpaso_destino, k, j: integer;
  F_inf, F_sup, alfa: NReal;
  aux_Vect: TVectR;
  funcionesDeformantesI: TDAOf_ddp_VectDeMuestras;
  res: TMatR;

begin
  arch_salida := arch + '111.txt';
  sintetizador_origen := TModeloSintetizadorCEGH.CreateFromArchi(arch);
  sintetizador_destino := TModeloSintetizadorCEGH.CreateFromArchi(arch);

  MatA := sintetizador_origen.A_cte.power_r(1.0 / paso_Discr);

  MatB := TMatR.Create_Init(sintetizador_origen.B_cte.nf, sintetizador_origen.B_cte.nc);
  MatB.Igual(sintetizador_origen.B_cte);

  sintetizador_destino.A_cte.Igual(MatA);

  res := TrasnformarB(MatA, MatB, paso_discr);
  if res <> nil then
  begin
    sintetizador_destino.B_cte.Igual(res);
    res.Free;
  end
  else
    raise Exception.Create( 'Fraccionar ... FALLO CHOLESKY al calcular B ' );

  // Interpolacion de los datos entre muestras de mayor frecuencia

  NSS_origen := sintetizador_origen.nBornesSalida;
  NFD_origen := length(sintetizador_origen.funcionesDeformantes[0]);
  NPFD_origen := sintetizador_origen.funcionesDeformantes[0][0].a.n;

  NSS_destino := NSS_origen;
  NFD_destino := NFD_origen * paso_discr;
  NPFD_destino := NPFD_origen;


  for kserie := 0 to NSS_origen - 1 do
  begin
    SetLength(sintetizador_destino.funcionesDeformantes[kserie],
      NFD_destino);
    funcionesDeformantesI :=
      sintetizador_destino.funcionesDeformantes[kserie];
    for kpaso := 1 to NFD_origen do
    begin
      aux_Vect := TVectR.Create_Init(NPFD_origen);
      for k := 1 to paso_discr do
      begin
        alfa := k / paso_discr;
        kpaso_destino := (kpaso - 1) * paso_discr + k - 1;

        for j := 0 to NPFD_origen do

        begin
          F_inf :=
            sintetizador_origen.funcionesDeformantes[kserie][kpaso - 1].a.e(j);
          if kpaso = NFD_origen then
          begin
            F_sup :=
              sintetizador_origen.funcionesDeformantes[kserie][0].a.e(j);
          end
          else
          begin
            F_sup :=
              sintetizador_origen.funcionesDeformantes[kserie][kpaso].a.e(j);
          end;

          aux_Vect.pon_e(
            j, Interpolacion_funciones_deformantes(alfa, F_inf, F_sup));

        end;
        funcionesDeformantesI[kpaso_destino] :=
          Tf_ddp_VectDeMuestras.Create_SinClonarMuestras(aux_Vect, nil, 0);
      end;
    end;

  end;

  sintetizador_destino.WriteToArchi(arch_salida, 0, 2);
  sintetizador_origen.NombresDeBornes_Publicados.Clear;
  sintetizador_destino.NombresDeBornes_Publicados.Clear;
  sintetizador_origen.Free;

end;



procedure muestra(arch: string; var a, b, c: integer);
var
  sintetizador: TModeloSintetizadorCEGH;
begin
  sintetizador := TModeloSintetizadorCEGH.CreateFromArchi(arch);
  a := sintetizador.nBornesSalida;
  b := sintetizador.nPuntosPorPeriodo;
  c := sintetizador.funcionesDeformantes[0][0].a.n;
  sintetizador.Free;
end;

end.
