unit uTSumComb;

interface

uses
  uCosa, SysUtils,
  uGlobs, uActores, uFechas, unodos, uCosaConNombre,
  uFichasLPD, usimplex, xMatDefs, uauxiliares,
  uActorNodalCombustible,
  unodocombustible,
  uFuentesAleatorias;

resourcestring
  rsSuministroSimpleCombustible = 'Suministro Simple Combustible';


type

  TSuministroCombustible = class(TActorUniNodalCombustible)
  end;

  { TFichaSuministroSimpleCombustible }

  TFichaSuministroSimpleCombustible = class(TFichaLPD)
  public
  (**************************************************************************)
  (*              A T R I B U T O S   P E R S I S T E N T E S               *)
  (**************************************************************************)

    QMax: NReal;   // m3/s
    flg_QMedMax_Activo: boolean; // Activa restricción de QMedMax
    QMedMax: NReal; // m3/s medios del paso MAXIMO.
    precio_por_unidad: NREal; //
    unidades_del_precio: string;
    flg_precio_a_PCS: boolean;
    fd: NReal;
    TMR: NReal;
    fuente_idxPrecioCombustible: TFuenteAleatoria;
    borne_idxPrecioCombustible: string;
  (**************************************************************************)

    kborne_idxPrecioCombustible: integer;

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      QMax: NReal;
      flg_QMedMax_Activo: boolean;
      QMedMax: NReal;
      Precio_por_unidad: NReal;
      unidades_del_precio: string; flg_precio_a_PCS: boolean;
      xFuente_idxPrecioCombustible: TFuenteAleatoria;
      xBorne_idxPrecioCombustible: string; fd, TMR: NReal);
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    function InfoAd_: string; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa:TClaseDeCosa; xVersion: Integer=-2): TDataColumnListOfCosa; override;
    {$ENDIF}

    published
    {$IFDEF MIGRACION_PERSISTENCIA}
      property _QMax: NReal                                   read QMax                        write QMax;
      property _flg_QMedMax_Activo: boolean                   read flg_QMedMax_Activo          write flg_QMedMax_Activo;
      property _QMedMax: NReal                                read QMedMax                     write QMedMax;
      property _precio_por_unidad: NREal                      read precio_por_unidad           write precio_por_unidad;
      property _unidades_del_precio: string                   read unidades_del_precio         write unidades_del_precio;
      property _flg_precio_a_PCS: boolean                     read flg_precio_a_PCS            write flg_precio_a_PCS;
      property _fd: NReal                                     read fd                          write fd;
      property _TMR: NReal                                    read TMR                         write TMR;
      property _fuente_idxPrecioCombustible: TFuenteAleatoria read fuente_idxPrecioCombustible write fuente_idxPrecioCombustible;
      property _borne_idxPrecioCombustible: string            read borne_idxPrecioCombustible  write borne_idxPrecioCombustible;
    {$ENDIF}

  end;

  { TSuministroSimpleCombustible }

  TSuministroSimpleCombustible = class(TSuministroCombustible)
  public
    // variables
    pa: TFichaSuministroSimpleCombustible;

    QMaxOperativo: NReal;
    PrecioOperativo_por_Q1h: NReal;
    NMaquinasDisponibles: integer;

    precio_USD_por_Q1h: NReal;

    constructor Create(capa: integer; nombre: string;
      nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD;
      lpd: TFichasLPD; NodoSumComb: TNodoCombustible);

    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    procedure PubliVars; override;
    procedure PrepararMemoria(globs: TGlobs); override;

    class function DescClase: string; override;

    procedure RegistrarParametrosDinamicos; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_cargue(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_leerSolucion(s: TSimplex); override;

    procedure CambioFichaPD; override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;

    procedure Free; override;

    procedure AfterInstantiation; override;

  end;

procedure AlInicio;
procedure ALFinal;

procedure TSuministroSimpleCombustible_CambioFichaPD( Actor: TCosa );


implementation


procedure TSuministroSimpleCombustible_CambioFichaPD( Actor: TCosa );
begin
  (Actor as TSuministroSimpleCombustible).cambioFichaPD;
end;

procedure TSuministroSimpleCombustible.CambioFichaPD;
begin
   precio_USD_por_Q1h:= NodoComb.combustible.precio_USD_por_MWh_a_PCI(
    pa.precio_por_unidad, pa.unidades_del_precio, pa.flg_precio_a_PCS)
    * NodoComb.combustible.MWh_por_Q1h_;
end;



// Métodos de TFichaSUministroSimpleCombustible

constructor TFichaSuministroSimpleCombustible.Create(capa: integer;
  fecha: TFecha; periodicidad: TPeriodicidad;
  QMax: NReal;
  flg_QMedMax_Activo: boolean;
  QMedMax: NReal;
  Precio_por_unidad: NReal;
  unidades_del_precio: string; flg_precio_a_PCS: boolean;
  xFuente_idxPrecioCombustible: TFuenteAleatoria; xBorne_idxPrecioCombustible: string;
  fd, TMR: NReal);
begin
  inherited Create(capa, fecha, periodicidad);
  self.Qmax := Qmax;
  self.flg_QMedMax_Activo:= flg_QMedMax_Activo;
  self.QMedMax:= QMedMax;
  self.precio_por_unidad := precio_por_unidad;
  self.unidades_del_precio:= unidades_del_precio;
  self.flg_precio_a_PCS:= flg_precio_a_PCS;
  self.fd := fd;
  self.TMR := TMR;
  self.fuente_idxPrecioCombustible := xFuente_idxPrecioCombustible;
  self.borne_idxPrecioCombustible := xBorne_idxPrecioCombustible;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TFichaSuministroSimpleCombustible.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('QMax', QMax);

  if f.Version >= 128 then
  begin
    f.rd('flg_QMedMax_Activo', flg_QMedMax_Activo );
    f.rd('QMedMax', QMedMax );
  end
  else
  begin
    flg_QMedMax_Activo:= false;
    QMedMax:= 0.0;
  end;

  f.rd('Precio_por_unidad', Precio_por_unidad);
  f.rd('unidades_del_precio', unidades_del_precio );
  f.rd('flg_precio_a_PCS', flg_precio_a_PCS );
  f.rd('fd', fd);
  f.rd('TMR', TMR);
  f.rdReferencia('fuente_idxPrecioCombustible',
    TCosa(fuente_idxPrecioCombustible), self);
  f.rd('borne_idxPrecioCombustible', borne_idxPrecioCombustible);
  f.EjecutarLectura;
end;

procedure TFichaSuministroSimpleCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('QMax', QMax);
  f.wr('flg_QMedMax_Activo', flg_QMedMax_Activo );
  f.wr('QMedMax', QMedMax );
  f.wr('Precio_por_unidad', Precio_por_unidad);
  f.wr('unidades_del_precio', unidades_del_precio );
  f.wr('flg_precio_a_PCS', flg_precio_a_PCS );
  f.wr('fd', fd);
  f.wr('TMR', TMR);
  f.wrReferencia('fuente_idxPrecioCombustible', fuente_idxPrecioCombustible);
  f.wr('borne_idxPrecioCombustible', borne_idxPrecioCombustible);
end;
{$ENDIF}

function TFichaSuministroSimpleCombustible.InfoAd_: string;
begin
  Result := 'QMax: ' + FloatToStr(QMax) + ', Precio['+unidades_del_precio+']: ' + FloatToStr(Precio_por_unidad);
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TFichaSuministroSimpleCombustible.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa,xVersion);
  Result.AddDoubleColumn('QMax', 0.0);
  Result.AddBooleanColumn('flg_QMedMax_Activo', False, 128);
  Result.AddDoubleColumn('QMedMax', 0.0, 128);
  Result.AddDoubleColumn('Precio_por_unidad', 0.0);
  Result.AddStringColumn('unidades_del_precio', '');
  Result.AddBooleanColumn('flg_precio_a_PCS', False);
  Result.AddDoubleColumn('fd', 0.0);
  Result.AddDoubleColumn('TMR', 0.0);
  Result.AddCosaColumn('fuente_idxPrecioCombustible');
  Result.AddStringColumn('borne_idxPrecioCombustible', '');
end;
{$ENDIF}


//--------------------------
// Métodos de TSuministroSimpleCombustible
//==========================

constructor TSuministroSimpleCombustible.Create(capa: integer;
  nombre: string; nacimiento, muerte: TFecha; lpdUnidades: TFichasLPD;
  lpd: TFichasLPD; NodoSumComb: TNodoCombustible);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, NodoSumComb);
  self.lpd := lpd;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TSuministroSimpleCombustible.Create_ReadFromText(f: TArchiTexto);
var
  a: Integer;
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('lpd', TCosa(lpd));
  f.EjecutarLectura;
  lpd.Propietario := self;
end;

procedure TSuministroSimpleCombustible.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('lpd', lpd);
end;
{$ENDIF}

procedure TSuministroSimpleCombustible.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableNR('Precio Operativo', '[USD/Q1h]', 6, 2, PrecioOperativo_por_Q1h, True);  //verificar unidades

end;

procedure TSuministroSimpleCombustible.PrepararMemoria(globs: TGlobs);
begin
  inherited PrepararMemoria(globs);
end;

class function TSuministroSimpleCombustible.DescClase: string;
begin
  Result := rsSuministroSimpleCombustible;
end;

procedure TSuministroSimpleCombustible.RegistrarParametrosDinamicos;
var
  i: integer;
  ficha: TFichaSuministroSimpleCombustible;

begin
  inherited registrarParametrosDinamicos;
  lpd.expandirFichas(globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil, TSuministroSimpleCombustible_CambioFichaPD );

  for i:= 0 to lpd.count - 1 do
  begin
    ficha := lpd[0] as TFichaSuministroSimpleCombustible;
    if ficha.fuente_idxPrecioCombustible <> nil then
      ficha.kborne_idxPrecioCombustible :=
        ficha.fuente_idxPrecioCombustible.IdBorne(ficha.borne_idxPrecioCombustible);
  end;
  pa:= lpd[0] as TFichaSuministroSimpleCombustible;
  CambioFichaPD;
end;


procedure TSuministroSimpleCombustible.SorteosDelPaso(sortear: boolean);
begin
  if hayForzamientos or globs.ObligarDisponibilidad_1_ then
  begin
    NMaquinasDisponibles := paUnidades.nUnidades_Operativas[0];;
    //supongo que nUnidades[0] tiene el total de maquinas de la central
    QMaxOperativo := pa.QMax * NMaquinasDisponibles;
  end
  else if sortear then
  begin
    ActualizarProbabilidadesReparacionYRotura_(pa.fd, pa.TMR);
    NMaquinasDisponibles := Sorteos_RepRotUnidades;
    QMaxOperativo := pa.QMax * NMaquinasDisponibles;
  end
  else
  begin
    NMaquinasDisponibles := paUnidades.nUnidades_Operativas[0];;
    //si no hay que sortear deja todas las maquinas disponibles
    QMaxOperativo := pa.QMax * pa.fd * NMaquinasDisponibles;
    //pero afecta a la potencia disponible por la disponibilidad
  end;

end;

procedure TSuministroSimpleCombustible.PrepararPaso_ps;
begin
  if pa.fuente_idxPrecioCombustible <> nil then
    PrecioOperativo_por_Q1h := precio_USD_por_Q1h * pa.fuente_idxPrecioCombustible.Bornera[
      pa.kborne_idxPrecioCombustible]
  else
    PrecioOperativo_por_Q1h := precio_USD_por_Q1h;
end;

procedure TSuministroSimpleCombustible.opt_nvers(var ivar, ivae, ires: integer);
begin
  self.ivar := ivar;
  Inc(ivar, globs.npostes);

  if pa.flg_QMedMax_Activo then
  begin
   self.ires:= ires;
   inc( ires );
  end;
end;

procedure TSuministroSimpleCombustible.opt_cargue(s: TSimplex);
var
  iresnodo: integer;
  kposte: integer;
begin
  iresnodo := self.NodoComb.ires;
  for kposte := 0 to globs.npostes - 1 do
  begin
    s.pon_e(iresnodo+ kposte, ivar + kposte, 1);

    // cargar funcion objetivo-costo
    s.pon_e(s.nf, ivar + kposte, -PrecioOperativo_por_Q1h * globs.durpos[kposte]);
  end;

  if pa.flg_QMedMax_Activo then
  begin
    for kposte := 0 to globs.npostes - 1 do
        s.pon_e( ires, ivar + kposte, -globs.DurPos[kposte] );
    s.pon_e( ires, s.nc, globs.HorasDelPaso * pa.QMedMax );
  end;

end;

procedure TSuministroSimpleCombustible.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  kposte: integer;
begin
  for kposte := 0 to globs.npostes - 1 do
    s.cota_sup_set(ivar + kposte, QMaxOperativo);


end;

procedure TSuministroSimpleCombustible.opt_leerSolucion(s: TSimplex);
var
  kposte: integer;
  Consumo: NREal;
begin
  Consumo := 0;
  for kposte := 0 to globs.npostes - 1 do
  begin
    Q[kposte] := s.xval(ivar + kposte);
    Consumo := Consumo + Q[kposte] * globs.DurPos[kposte];
  end;

  CostoDirectoDelPaso := Consumo * PrecioOperativo_por_Q1h;
end;

function TSuministroSimpleCombustible.getNombreVar(ivar: integer; var nombre: string): boolean;
begin
  if (ivar >= self.ivar) and (ivar < self.ivar + globs.NPostes) then
  begin
    nombre := self.Nombre + '_Q[m3/s]' + IntToStr(ivar - self.ivar + 1);
    Result := True;
  end
  else
    Result := False;
end;


procedure TSuministroSimpleCombustible.Free;
begin
  inherited Free;
end;

procedure TSuministroSimpleCombustible.AfterInstantiation;
begin
  inherited AfterInstantiation;
  lpd.Propietario := self;
end;

procedure AlInicio;
begin
  registrarClaseDeCosa(TSuministroSimpleCombustible.ClassName,
    TSuministroSimpleCombustible);
  registrarClaseDeCosa(TFichaSuministroSimpleCombustible.ClassName,
    TFichaSuministroSimpleCombustible);

end;

procedure AlFinal;
begin
end;

end.
