unit uConversions;
interface
{$IFDEF MIGRACION_PERSISTENCIA}
uses
  Classes, SysUtils, xmatdefs, typinfo;

const
  VERSION_MIGRACION_PERSISTENCIA=140;

type

  TClassOfObject = class of TObject;

  TConversionFunction = function(input: array of string):TDAofString of object;

  TListDataConversion=class;

  { TDataConversion }

  TDataConversion=class
    Version:       integer;
    ColumnsInput:  TDAofString;
    ColumnsOutput: TDAofString;
    Convert:       TConversionFunction;

    Owner:TListDataConversion;

    constructor Create(xVersion: integer; xColumnsInput: array of string; xColumnsOutput: array of string; xConvert: TConversionFunction);
    procedure Free;
  end;

  { TDataConversionMultiTable }

  TDataConversionMultiTable=class(TDataConversion)
  public
    ColumnsInputCorrelation: TDAofString;
    constructor Create(xVersion:                 integer;
                       xColumnsInput:            array of string;
                       xColumnsOutput:           array of string;
                       xColumnsInputCorrelation: array of string);
    function ConversionMULTITABLE(input:array of string): TDAofString;
  end;

  { TListDataConversion }

  TListDataConversion=class(TList)
  private
    function ArithmeticOperation(AOperator:string; input:array of string): TDAofString;

  protected
    function GetItem(Index: Integer): TDataConversion;
  public

    ClassOfObject: TClassOfObject;

    function Add(xVersion: integer; xColumnsInput: array of string; xColumnsOutput: array of string; xConvert: TConversionFunction):integer; overload;
    function Add(xVersion:                 integer;
                 xColumnsInput:            array of string;
                 xColumnOutput:            string;
                 xColumnsInputCorrelation: array of string): integer; overload;

    property Items[Index: Integer]: TDataConversion read GetItem;
    procedure Free;

    (*** ARITHMETIC OPERATIONS ***)

    function ConversionADDITION       (input:array of string): TDAofString;
    function ConversionSUBTRACTION    (input:array of string): TDAofString;
    function ConversionMULTIPLICATION (input:array of string): TDAofString;
    function ConversionDIVISION       (input:array of string): TDAofString;

    function ConversionCOPYVALUE      (input:array of string): TDAofString;

    (*** STRING OPERATIONS ***)
    function ConversionNAMECHANGE (input:array of string): TDAofString;

    (*** PRIMITIVE TO DYNARRAY CONVERSION ***)

    // Crea un array de largo uno con el valor del primitivo
    function ConversionPrimitiveToDynArray (input:array of string): TDAofString;

    // Crea un array de largo uno con el valor del primitivo
    function ConversionPrimitivesToOneDynArray (input:array of string): TDAofString;

    // el input es el array referencia para obtener el largo
    function ConversionDynArrayInitOnes(input:array of string): TDAofString;

  end;

  //Recibe un array [5| a, d, s, f, q] y devuelve 5
  // Si se le pasa la lista items devuelve las entradas del array
  function GetStrDynArrayCount(strDynArr: string):Integer;
  function DynArrayInit(Count: integer; Value: string):string;
{$ENDIF}
implementation
{$IFDEF MIGRACION_PERSISTENCIA}

uses
  ucosa;

function GetStrDynArrayCount(strDynArr: string): Integer;
var
  split: TStringList;
  i: Integer;
begin
  try
    split:= TStringList.Create;
    ExtractStrings(['[','|',',',']'], [], PChar(strDynArr), split);
    Result := StrToInt( split[0] );
  except
    raise Exception.Create('Se espera un array con "|" como separador del largo y "," como separador de elementos. Se recibió '+strDynArr);
  end;
  split.Free;

end;

function DynArrayInit(Count: integer; Value: string): string;
var
  i: Integer;
begin
  Result:='['+IntToStr(Count)+'| ';
  for i:=0 to Count-2 do
    Result:=Result+Value+', ';
  Result:=Result+Value+']';
end;

{ TDataConversionMultiTable }

constructor TDataConversionMultiTable.Create(xVersion: integer;
  xColumnsInput: array of string; xColumnsOutput: array of string;
  xColumnsInputCorrelation: array of string);
var
  i: Integer;
begin
  inherited Create(xVersion, xColumnsInput, xColumnsOutput, nil);

  self.Convert:=self.ConversionMULTITABLE;

  SetLength(self.ColumnsInputCorrelation, Length(xColumnsInputCorrelation));
  for i:= Low(xColumnsInputCorrelation) to High(xColumnsInputCorrelation) do
    self.ColumnsInputCorrelation[i]:=xColumnsInputCorrelation[i];

end;

function TDataConversionMultiTable.ConversionMULTITABLE(input: array of string
  ): TDAofString;
var
  PropInfo: PPropInfo;
  ClassOfOutputColumn: TClassOfObject;
  DataTableName: String;
  DataConversionList: TListDataConversion;
  b: String;
  a: Integer;
  elementsAreEmpty: Boolean;
  i: Integer;
  DataTable: TDataTableOfCosa;
  index: Integer;
  DataRow: TDataRowOfCosa;
begin

  SetLength(Result, 1);

  elementsAreEmpty:=true;
  for i:=Low(input) to High(input) do
    if input[i]<>'' then
    begin
      elementsAreEmpty:=false;
      break;
    end;

  if not elementsAreEmpty then
  begin
    DataConversionList:=self.Owner;

    PropInfo:=typinfo.FindPropInfo(DataConversionList.ClassOfObject.ClassType, '_'+self.ColumnsOutput[0]);
    if Assigned(PropInfo) then
    begin
      ClassOfOutputColumn:=TClassOfObject(typinfo.GetTypeData(PropInfo^.PropType)^.ClassType);
      DataTableName:='dt_'+ClassOfOutputColumn.ClassName;

      DataTable:=DATA_STORE.GetDataTableByName(DataTableName);
      if not Assigned(DataTable) then
        DataTable:=TDataTableOfCosa.Create(DataTableName, TClaseDeCosa(ClassOfOutputColumn));

      DataRow:=TDataRowOfCosa.Create();

      for i:=Low(self.ColumnsInputCorrelation) to High(self.ColumnsInputCorrelation) do
        WriteLn(IntToStr(i)+'-'+self.ColumnsInputCorrelation[i]+'='+input[i]);

      for i:=0 to DataTable.DataColumnListCount-1 do
        WriteLn(IntToStr(DataTable.DataColumnList[i].Index)+'-'+DataTable.DataColumnList[i].ColumnName);

      for i:=0 to DataTable.DataColumnListCount-1 do
        DataRow.AddDataCell(TDataCellOfCosa.Create(DataTable.DataColumnList[i].ValorPorDefecto));

      for i:=Low(self.ColumnsInputCorrelation) to High(self.ColumnsInputCorrelation) do
      begin
        b:=self.ColumnsInputCorrelation[i];
        index:=DataTable.GetIndexOfDataColumnByName(self.ColumnsInputCorrelation[i]);
        if index>-1 then
        begin
          DataRow.DataCellList[index].sval:=input[i];
        end;
      end;

      Result[0]:=DataTableName+'.'+IntToStr(DataTable.AddDataRow(DataRow));
    end;
  end
  else
    Result[0]:='';
end;

function TListDataConversion.ArithmeticOperation(AOperator: string; input: array of string
  ): TDAofString;
var
  i: Integer;
begin
  SetLength(Result, 1);
  Result[0]:='"';
  for i:=Low(input) to High(input)-1 do
    Result[0]:=Result[0] + input[i] + AOperator;
  Result[0]:=Result[0] + input[High(input)] + '"';
end;

function TListDataConversion.ConversionADDITION(input: array of string): TDAofString;
begin
  Result:=ArithmeticOperation('+', input);
end;

function TListDataConversion.ConversionSUBTRACTION(input: array of string): TDAofString;
begin
  Result:=ArithmeticOperation('-', input);
end;

function TListDataConversion.ConversionMULTIPLICATION(input: array of string): TDAofString;
begin
  Result:=ArithmeticOperation('*', input);
end;

function TListDataConversion.ConversionDIVISION(input: array of string): TDAofString;
begin
  Result:=ArithmeticOperation('/', input);
end;

function TListDataConversion.ConversionCOPYVALUE(input: array of string
  ): TDAofString;
var
  i: Integer;
  b:Boolean;
begin
  SetLength(Result, Length(input));
  for i:=low(input) to high(input) do
    Result[i]:=input[i];
end;

function TListDataConversion.ConversionNAMECHANGE(input: array of string): TDAofString;
var
  i: Integer;
begin
  SetLength(Result, Length(input));
  for i:=Low(input) to High(input) do
    Result[i]:=input[i];
end;

function TListDataConversion.ConversionPrimitiveToDynArray(
  input: array of string): TDAofString;
var
  i: Integer;
begin
  SetLength(Result, Length(input));
  for i:=Low(input) to High(input) do
    Result[i]:='[1| '+input[i]+']';
end;

function TListDataConversion.ConversionPrimitivesToOneDynArray(
  input: array of string): TDAofString;
var
  i: Integer;
begin
  SetLength(Result, 1);
  Result[0]:='['+IntToStr(Length(input))+'| ';
  for i:=Low(input) to High(input)-1 do
    Result[0]:=Result[0]+input[i]+', ';

  Result[0]:=Result[0]+input[High(input)]+']';

end;

function TListDataConversion.ConversionDynArrayInitOnes(input: array of string
  ): TDAofString;
begin
  SetLength(Result,1);
  Result[0]:=DynArrayInit(Length(input), '1');
end;

{ TListDataConversion }

function TListDataConversion.GetItem(Index: Integer): TDataConversion;
begin
   Result:=TDataConversion(inherited Items[Index]);
end;

function TListDataConversion.Add(xVersion: integer;
  xColumnsInput: array of string; xColumnsOutput: array of string;
  xConvert: TConversionFunction): integer;
begin
  Result:=inherited Add(TDataConversion.Create(xVersion, xColumnsInput, xColumnsOutput, xConvert));
  self.Items[Result].Owner:=self;
end;

function TListDataConversion.Add(xVersion: integer;
                                 xColumnsInput: array of string;
                                 xColumnOutput: string;
                                 xColumnsInputCorrelation: array of string): integer;
begin
  Result:=inherited Add(TDataConversionMultiTable.Create(xVersion,
                                                         xColumnsInput,
                                                         [xColumnOutput],
                                                         xColumnsInputCorrelation));
  self.Items[Result].Owner:=self;
end;

procedure TListDataConversion.Free;
var
  i: Integer;
begin
  for i:=0 to Count-1 do
    Items[i].Free;

  inherited Free;
end;

{ TDataConversion }

constructor TDataConversion.Create(xVersion: integer;
  xColumnsInput: array of string; xColumnsOutput: array of string;
  xConvert: TConversionFunction);
var
  i: Integer;
begin
  self.Version:=xVersion;

  SetLength(self.ColumnsInput, Length(xColumnsInput));
  for i:= Low(xColumnsInput) to High(xColumnsInput) do
    self.ColumnsInput[i]:=xColumnsInput[i];

  SetLength(self.ColumnsOutput, Length(xColumnsOutput));
  for i:= Low(xColumnsOutput) to High(xColumnsOutput) do
    self.ColumnsOutput[i]:=xColumnsOutput[i];

  self.Convert:=xConvert;
end;

procedure TDataConversion.Free;
begin
  SetLength(ColumnsInput,0);
  SetLength(ColumnsOutput,0);
  SetLength(ColumnsInput,0);
  Convert:=nil;

  inherited Free;

end;
{$ENDIF}
end.


