unit uReferenciaMonSimRes;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
  uReferenciaMonitor, uEventosOptSim, uVarDefs, uMonitorSimRes, uCosa,
  uCosaConNombre, uMonitores, uSalasDeJuego, Classes;

resourcestring
  rsMonitorDeArchivoSimRes = 'Monitor de Archivo Para SimRes';
  mesEnElMonitor = 'En el monitor ';

type

  { TReferenciaMonSimRes }

  TReferenciaMonSimRes = class(TReferenciaMonMultiVar)
  public
    sala: TSalaDeJuego;
    filename: string;

    constructor Create(capa: integer; nombreMon: string; filename: string;
      defVars: TListaDeCosas; sala: TSalaDeJuego);
      reintroduce;
    {$IFNDEF MIGRACION_PERSISTENCIA}
    constructor Create_ReadFromText(f: TArchiTexto); override;
    procedure WriteToText(f: TArchiTexto); override;
    {$ENDIF}
    class function DescClase: string; override;
    procedure crearMonitorDeVariables(cosasMonitoreables: TListaDeCosasConNombre);
      override;
    class function tiposMonitoreables: AOfVarDef; override;

    {$IFDEF MIGRACION_PERSISTENCIA}
    class function CreateDataColumnList(xClaseDeCosa: TClaseDeCosa; xVersion: Integer=-2
      ): TDataColumnListOfCosa; override;

  published
      property _sala: TSalaDeJuego read sala     write sala;
      property _filename: string   read filename write filename;
    {$ENDIF}

  end;

procedure AlInicio;
procedure AlFinal;

var
  VarDefsValidas_TMonSimRes: AOfVarDef;

implementation

constructor TReferenciaMonSimRes.Create(capa: integer; nombreMon: string;
  filename: string; defVars: TListaDeCosas {of TReferenciaDefVar}; sala: TSalaDeJuego);
var
  xEventosProcs: TDAOfEventoProc;
begin
  SetLength(xEventosProcs, 4);
  xEventosProcs[0].evento := E_Sim_Inicio;
  xEventosProcs[0].refProc := TMonSimRes.darRefProc('AbrirArchivo', E_Sim_Inicio);
  xEventosProcs[1].evento := E_Sim_InicioCronica;
  xEventosProcs[1].refProc := TMonSimRes.darRefProc('EscribirEncabezadoCronica',
    E_Sim_InicioCronica);
  xEventosProcs[2].evento := E_Sim_FinPaso;
  xEventosProcs[2].refProc := TMonSimRes.darRefProc('EscribirPaso', E_Sim_FinPaso);
  xEventosProcs[3].evento := E_Sim_Fin;
  xEventosProcs[3].refProc := TMonSimRes.darRefProc('CerrarArchivo', E_Sim_Fin);

  inherited Create(capa, nombreMon, xEventosProcs, defVars);
  Self.filename := filename;
  self.sala := sala;
end;

{$IFNDEF MIGRACION_PERSISTENCIA}
constructor TReferenciaMonSimRes.Create_ReadFromText(f: TArchiTexto);
begin
  inherited Create_ReadFromText(f);
  f.IniciarLecturaRetrasada;
  f.rd('filename', filename);
  f.rdReferencia('sala', TCosa(sala), self);
  f.EjecutarLectura;
end;
{$ENDIF}

{$IFNDEF MIGRACION_PERSISTENCIA}
procedure TReferenciaMonSimRes.WriteToText(f: TArchiTexto);
begin
  inherited WriteToText(f);
  f.wr('filename', filename);
  f.wrReferencia('sala', sala);
end;
{$ENDIF}

class function TReferenciaMonSimRes.DescClase: string;
begin
  Result := rsMonitorDeArchivoSimRes;
end;

procedure TReferenciaMonSimRes.crearMonitorDeVariables(cosasMonitoreables:
  TListaDeCosasConNombre);
var
  ListaDefVars: TList; {of TVarDef}
  i: integer;
  varDef: TVarDef;
  errores: string;
begin
  eventos := sortTDAOfEventoProcByProc(eventos);
  errores := '';
  ListaDefVars := TList.Create;
  ListaDefVars.Capacity := defVars.Count;
  for i := 0 to defVars.Count - 1 do
  begin
    try
      begin
        varDef := TReferenciaDefVar(defVars[i]).resolver(cosasMonitoreables);
        varDef.Prepararse;
        ListaDefVars.Add(varDef);
      end
    except
      on E: EReferenciaDefVarException do
      begin
        errores := errores + E.Message;
      end
    end;
  end;
  i := ListaDefVars.Count;
  if ListaDefVars.Count <> 0 then
    monitor := TMonSimRes.Create(ListaDefVars, filename, sala);
  if errores <> '' then
    raise EMonitorException.Create(errores + mesEnElMonitor + nombre + '.', i);
end;

class function TReferenciaMonSimRes.tiposMonitoreables: AOfVarDef;
begin
  Result := VarDefsValidas_TMonSimRes;
end;

{$IFDEF MIGRACION_PERSISTENCIA}
class function TReferenciaMonSimRes.CreateDataColumnList(
  xClaseDeCosa: TClaseDeCosa; xVersion: Integer): TDataColumnListOfCosa;
begin
  Result:=inherited CreateDataColumnList(xClaseDeCosa, xVersion);
  Result.AddStringColumn('filename', '');
  Result.AddCosaColumn('sala');
end;
{$ENDIF}

procedure AlInicio;
begin
  registrarClaseDeCosa(TReferenciaMonSimRes.ClassName, TReferenciaMonSimRes);

  SetLength(VarDefsValidas_TMonSimRes, 5);
  VarDefsValidas_TMonSimRes[0] := TVarDef_NR;
  VarDefsValidas_TMonSimRes[1] := TVarDef_NI;
  VarDefsValidas_TMonSimRes[2] := TVarDef_Fecha;
  VarDefsValidas_TMonSimRes[3] := TVarDef_VNR;
  VarDefsValidas_TMonSimRes[4] := TVarDef_VNI;
end;

procedure AlFinal;
begin
  SetLength(VarDefsValidas_TMonSimRes, 0);
end;

end.
